Package: DNAshapeR
Type: Package
Title: High-throughput prediction of DNA shape features
Version: 1.41.0
Date: 2018-06-08
Author: Tsu-Pei Chiu and Federico Comoglio
Maintainer: Tsu-Pei Chiu <tsupeich@usc.edu>
Description: DNAhapeR is an R/BioConductor package for ultra-fast,
        high-throughput predictions of DNA shape features. The package
        allows to predict, visualize and encode DNA shape features for
        statistical learning.
License: GPL-2
biocViews: StructuralPrediction, DNA3DStructure, Software
Depends: R (>= 3.4), GenomicRanges
Imports: Rcpp (>= 0.12.1), Biostrings, fields
LinkingTo: Rcpp
NeedsCompilation: yes
Packaged: 2026-05-30 08:48:15 UTC; root
VignetteBuilder: knitr
Suggests: AnnotationHub, knitr, rmarkdown, testthat,
        BSgenome.Scerevisiae.UCSC.sacCer3, BSgenome.Hsapiens.UCSC.hg19,
        caret
RoxygenNote: 6.0.1
Config/pak/sysreqs: zlib1g-dev
Repository: https://bioc.r-universe.dev
Date/Publication: 2026-04-28 12:43:02 UTC
RemoteUrl: https://github.com/bioc/DNAshapeR
RemoteRef: HEAD
RemoteSha: d2b3810232e86916f6064c68fdd85c319c52d566
Built: R 4.6.0; x86_64-apple-darwin20; 2026-05-30 08:54:14 UTC; unix

import(GenomicRanges)

importFrom(Rcpp, evalCpp)
importFrom(Biostrings, readDNAStringSet)
importFrom(Biostrings, mkAllStrings)
importFrom(Biostrings, writeXStringSet)
importFrom(Biostrings, getSeq)

importFrom("grDevices", "colorRampPalette", "rgb")
importFrom("graphics", "abline", "axis", "barplot", "lines", "par", "plot", "points")
importFrom("stats", "lowess", "sd")


importFrom(fields, image.plot)
useDynLib(DNAshapeR)

export(
    encodeSeqShape,
    getFasta,
    getShape,
    heatShape,
    readShape,
    plotShape,
    trackShape
)

DNAshapeR
----

DNAshapeR is a R/BioConductor package for DNA shape prediction and feature encoding.

The latest (devel) version of DNAshapeR requires [R >3.2.2](https://cran.r-project.org) and can be installed from this GitHub repository with:

```R
 library(devtools)
 install_github(repo = "TsuPeiChiu/DNAshapeR", build_vignettes = TRUE)
```

For more information, package vignette and documentation, please visit [http://tsupeichiu.github.io/DNAshapeR/](http://tsupeichiu.github.io/DNAshapeR/)

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# DNAshapeR
# 2015
# Tsu-Pei Chiu, Rohs Lab, USC
# Federico Comoglio, Green lab, CIMR
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#' Plot metaprofiles of DNA shape features
#'
#' DNA shape features can be visualized as
#' aggregated line plots (also known as metaprofiles, see Comoglio et al., 2015), heat maps (Yang et al., 2014) and
#' genome browser tracks (Chiu et al., 2014).
#'
#' @usage plotShape(shapeMatrix, background = NULL,
#' colDots = rgb( 0, 0, 1, 0.1),
#' colDotsBg = rgb( 0, 0, 0, 0.1),
#' colLine = 'steelblue', colLineBg = 'gray50', cex = 0.5, lwd = 2, ylim, ...)
#'
#' @param shapeMatrix A matrix containing DNAshape prediction results
#' @param background A matrix containing DNAshape prediction results for a set of background regions. Default to NULL, i.e. background not provided.
#' @param colDots A character vector specifying the color of the points representing the column mean of shapeMatrix. Default to rgb( 0, 0, 1, 0.1).
#' @param colDotsBg A character vector specifying the color of the points representing the column mean of background. Default to rgb( 0, 0, 0, 0.1).
#' @param colLine A character string giving the color name of line representing the column mean of shapeMatrix. Default to 'steelblue'.
#' @param colLineBg A character string giving the color name of line representing the column mean of background. Default to 'gray50'.
#' @param cex A numerical value giving the amount by which plotting text and symbols should be magnified relative to the default. Default to 0.5.
#' @param lwd A numerical value specifying the line width. Default to 2.
#' @param ylim A numerical vector of size 2 specifying the y-axis plot range.
#' @param ... Additional parameters to be passed to the R plot function.
#'
#' @return Called for its effects
#'
#' @author Federico Comoglio
#'
#' @keywords core
#'
#' @examples
#' fn <- system.file("extdata", "CGRsample.fa", package = "DNAshapeR")
#' pred <- getShape(fn)
#' plotShape(pred$MGW)
#' plotShape(pred$ProT)
#' plotShape(pred$Roll)
#' plotShape(pred$HelT)
#' @export plotShape

plotShape <- function( shapeMatrix, background = NULL,
                        colDots = rgb( 0, 0, 1, 0.1),
                        colDotsBg = rgb( 0, 0, 0, 0.1),
                        colLine = 'steelblue',
                        colLineBg = 'gray50', cex = 0.5, lwd = 2, ylim, ... ) {
    n <- nrow( shapeMatrix )
    mu <- colMeans( shapeMatrix, na.rm = TRUE )
    m <- length( mu )
    span <- round( m / 2)

    if( is.null( background ) ) {
        if( missing( ylim ) )
			ylim <- range( mu, na.rm = TRUE )

        plot( mu,
            col = colDots,
            pch = 19,
            cex = cex,
            xaxt = 'n',
            xlab = '',
            ylab = paste0( 'Mean value (n=', n, ')'),
            ylim = ylim,
            ... )
        axis( 1, at = c( 0, span, m ),
          labels = c( -span, 'Center', paste0( '+', span ) ) )
          abline( v = span, lty = 2, col = 'gray30' )
          lines( lowess( mu, f = 1/10 ), col = colLine, lwd = lwd )
    }

    else { #shape of random sample is provided
        mu1 <- mu
        mu2 <- colMeans( background, na.rm = TRUE )

        if( missing( ylim ) )
			ylim <- range( mu1, mu2, na.rm = TRUE )

        plot( mu1,
            col = colDots,
            pch = 19,
            cex = cex,
            xaxt = 'n',
            xlab = '',
            ylab = paste0( 'Mean value (n=', n, ')'),
            ylim = ylim,
            ... )

        points( mu2, pch = 19, cex = cex, col = colDotsBg )
        axis( 1, at = c( 0, span, m ),
            labels = c( -span, 'Center', paste0( '+', span ) ) )
        abline( v = span, lty = 2, col = 'gray30' )
        lines( lowess( mu1, f = 1/10 ), col = colLine, lwd = lwd )
        lines( lowess( mu2, f = 1/10 ), col = colLineBg, lwd = lwd )
    }
}

#' Plot heatmap of DNA shape features
#'
#' @usage heatShape(shapeMatrix, nBins, ordRow = NULL, useRaster = TRUE, ... )
#'
#' @param shapeMatrix A matrix containing DNAshape prediction results.
#' @param nBins An integer specifying the number of equally-sized bins in which shape predictions should be aggregated. Summarized predictions can be visualized by setting nBins=1.
#' @param ordRow A numeric vector (of the same length as the number of rows of shapeMatrix) defining the permutation of the rows of shapeMatrix to be used for plotting. Default to NULL, i.e. rows are ordered by coefficients of variation.
#' @param useRaster Logical, if TRUE a bitmap raster is used to plot the image instead of polygons (see ?graphics::image for details).
#' @param ... Additional parameters to be passed to the image.plot function (see ?fields::image.plot for details).
#'
#' @return Called for its effects
#'
#' @author Federico Comoglio
#'
#' @keywords core
#'
#' @examples
#' fn <- system.file("extdata", "CGRsample.fa", package = "DNAshapeR")
#' pred <- getShape(fn)
#' library(fields)
#' heatShape(pred$MGW, 20)
#' @export heatShape

heatShape <- function( shapeMatrix, nBins, ordRow = NULL,
                        useRaster = TRUE, ... ) {
    nc <- ncol( shapeMatrix )
    nr <- nrow( shapeMatrix )

    if( (nc %% nBins) != 0 )
        stop( 'The number of bases must be a multiple of the number of bins.' )

    d <- nc / nBins
    by <- rep( 1 : nBins, each = d * nr )
    matByBins <- split( shapeMatrix, by)
    matByBins <- lapply( matByBins,
                    function(x) matrix( x, ncol = d, byrow = FALSE ) )
    bins <- do.call( 'cbind', lapply( matByBins, rowMeans, na.rm = TRUE ) )

    if( is.null( ordRow ) ) {
        sdRow <- apply( bins, 1, sd )
        meanRow <- rowMeans( bins )
        cvRow <- abs( sdRow/meanRow )
        ordRow <- order( cvRow, decreasing = FALSE )
    }

    palCol <- c( 'navyblue', 'white', 'red4' )
    pal <- colorRampPalette( palCol )( 64 )
    span <- round( nc / 2 )

    image.plot( t( bins[ rev( ordRow ), ] ), axes = FALSE, col = pal,
        legend.shrink = 0.2, useRaster = useRaster, ... )
    abline(v = 0.5, lwd = 2, lty = 2, col = "white")
    axis( 1, at = c( 0, 0.5, 1 ),
        labels = c( -span, 'Center', paste0( '+', span ) ) )
}

# 2015 - Tsu-Pei Chiu, Rohs Lab, USC
#'
#' Plot track view of DNA shape features
#'
#' @usage trackShape( filename, shapeList )
#'
#' @param filename The name of the input fasta format file, including
#' full path to file if it is located outside the current working directory
#' @param shapeList A list containing four DNAshape prediction results
#'
#' @return Called for its effects
#'
#' @note None.
#'
#' @author Tsu-Pei Chiu
#'
#' @keywords core
#'
#' @examples
#' fn2 <- system.file("extdata", "SingleSeqsample.fa", package = "DNAshapeR")
#' pred2 <- getShape(fn2)
#' trackShape(fn2, pred2) # Only for single sequence file
#' @export trackShape

trackShape <- function( filename, shapeList ) {

    records <- scan( filename, what = 'character' )
    recordStart <- grep( '>', records )

    if( length( recordStart ) > 1 ) { #multiple records
        stop("Only one sequence file is allowed!")

    }else{
        seq <- unlist(strsplit(scan(filename,
            what="character", sep="" , comment.char = ">" ), ""))

        mgw <- as.vector( shapeList$MGW )
        prot <- as.vector( shapeList$ProT )
        roll <- as.vector( shapeList$Roll )
        helt <- as.vector( shapeList$HelT )

        mgw <- mgw-2.5
        helt <- helt-30

        mgw[is.na(mgw)] <- 0
        prot[is.na(prot)] <- 0
        roll[is.na(roll)] <- 0
        helt[is.na(helt)] <- 0

        roll <- append(roll, 0, after = length(roll)-1)
        helt <- append(helt, 0, after = length(helt)-1)

        mydf <- data.frame(mgw, prot, roll, helt)
        #print(mydf)

        pos = seq(1, length(seq), 1)
        pos2 = pos+0.5


        par( mfrow = c( 4, 1), oma=c(2,2,2,2), mar=c(4,5,2,2),
            cex.axis = 1.25, cex.lab = 1.25)

        barplot(mydf$mgw, space = 0, col = "#F59547", border = NA,
            names.arg = seq, ylab = 'MGW (angstrom)',
                cex.names = 1.2, offset = 2.5 )
        barplot(mydf$prot, space = 0, col = "#99BA5B", border = NA,
            names.arg = seq, ylab = 'ProT (degree)',
                cex.names = 1.2 )

        rollP <- barplot(mydf$roll, space = 0, col = "#A1ACE6", border = NA,
            ylab = 'Roll (degree)', cex.names = 1.2 )
                axis(1, at=rollP-(rollP[2]-rollP[1])/2,las=2,
                labels=seq, tick = FALSE, las=0)

        helTP <- barplot(mydf$helt, space = 0, col = "#7F64A1", border = NA,
                ylab = 'HelT (degree)', cex.names = 1.2, offset = 30 )
        axis(1, at=helTP-(helTP[2]-helTP[1])/2,las=2,
                labels=seq, tick = FALSE, las=0)
    }
}

## ----eval=TRUE----------------------------------------------------------------
library(DNAshapeR)

## ----eval=TRUE----------------------------------------------------------------
library(DNAshapeR)
fn <- system.file("extdata", "CGRsample.fa", package = "DNAshapeR")
pred <- getShape(fn)

## ----eval=FALSE---------------------------------------------------------------
# # Install Bioconductor packages
# if (!requireNamespace("BiocManager", quietly=TRUE))
#     install.packages("BiocManager")
# BiocManager::install("BSgenome.Scerevisiae.UCSC.sacCer3")
# 
# library(BSgenome.Scerevisiae.UCSC.sacCer3)
# 
# # Create a query GRanges object
# gr <- GRanges(seqnames = c("chrI"),
#             strand = c("+", "-", "+"),
#             ranges = IRanges(start = c(100, 200, 300), width = 100))
# getFasta(gr, Scerevisiae, width = 100, filename = "tmp.fa")
# fn <- "tmp.fa"
# pred <- getShape(fn)

## ----eval=FALSE---------------------------------------------------------------
# # Install Bioconductor packages
# library(BSgenome.Hsapiens.UCSC.hg19)
# library(AnnotationHub)
# 
# ah <- AnnotationHub()
# ah <- subset(ah, species=="Homo sapiens")
# ah <- query(ah, c("H3K4me3", "Gm12878", "Roadmap"))
# getFasta(ah[[1]], Hsapiens, width = 150, filename = "tmp.fa")
# fn <- "tmp.fa"
# pred <- getShape(fn)

## ----eval=TRUE----------------------------------------------------------------
library(DNAshapeR)
fn_methy <- system.file("extdata", "MethylSample.fa", package = "DNAshapeR")
pred_methy <- getShape(fn_methy, methylate = TRUE)
pred_methy$MGW

## ----eval=TRUE----------------------------------------------------------------
library(DNAshapeR)
fn_methy <- system.file("extdata", "SingleSeqsample.fa", package = "DNAshapeR")
fn_methy_pos <- system.file("extdata", "MethylSamplePos.fa", package = "DNAshapeR")
pred_methy <- getShape(fn_methy, methylate = TRUE, methylatedPosFile = fn_methy_pos)
pred_methy$MGW

## ----fig.width=7, fig.height=7, fig.align='center', eval=TRUE-----------------
plotShape(pred$MGW)
#plotShape(pred$ProT)
#plotShape(pred$Roll)
#plotShape(pred$HelT)

## ----fig.width=7, fig.height=7, fig.align='center', eval=TRUE-----------------
library(fields)
heatShape(pred$ProT, 20)
#heatShape(pred$MGW, 20)
#heatShape(pred$Roll[1:500, 1:1980], 20)
#heatShape(pred$HelT[1:500, 1:1980], 20)

## ----fig.width=7, fig.height=7, fig.align='center', eval=TRUE-----------------
fn2 <- system.file("extdata", "SingleSeqsample.fa", package = "DNAshapeR")
pred2 <- getShape(fn2)
trackShape(fn2, pred2) # Only for single sequence file

## ----eval=TRUE----------------------------------------------------------------
library(Biostrings)
fn3 <- system.file("extdata", "PBMsample_short.fa", package = "DNAshapeR")
pred3 <- getShape(fn3)
featureType <- c("1-mer", "1-shape")
featureVector <- encodeSeqShape(fn3, pred3, featureType)
head(featureVector)

## ----eval=TRUE----------------------------------------------------------------
fn4 <- system.file("extdata", "PBMsample_short.s", package = "DNAshapeR")
experimentalData <- read.table(fn4)
df <- data.frame(affinity=experimentalData$V1, featureVector)

## ----eval=TRUE----------------------------------------------------------------
library(caret)

trainControl <- trainControl(method = "cv", number = 3, 
                savePredictions = TRUE)
model <- train (affinity~ ., data = df, 
                trControl=trainControl, method="lm", preProcess=NULL)
model

## ----eval=TRUE----------------------------------------------------------------
sessionInfo()

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# DNAshapeR
# 2015, 2017
# Tsu-Pei Chiu, Rohs Lab, USC
# Federico Comoglio, Green lab, CIMR
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#' Predict DNA shape from a FASTA file
#'
#' The DNA prediction uses a sliding pentamer window where structural features
#' unique to each of the 512 distinct pentamers define a vector of minor
#' groove width (MGW), Roll, propeller twist (ProT), and helix twist (HelT) at
#' each nucleotide position (Zhou, et al., 2013). MGW and ProT define
#' base-pair parameter whereas Roll and HelT represent base pair-step
#' parameters. The values for each DNA shape feature as function of its
#' pentamer sequence were derived from all-atom Monte Carlo simulations where
#' DNA structure is sampled in collective and internal degrees of freedom in
#' combination with explicit counter ions (Zhang, et al., 2014). The Monte
#' Carlo simulations were analyzed with a modified Curves approach
#' (Zhou, et al., 2013). Through data mining, average values for each shape
#' feature were calculated for the on average 44 occurrences of each pentamer
#' in an ensemble of Monte Carlo trajectories for 2,121 DNA fragments of 12-27
#' base pairs in length. DNAshapeR predicts four DNA shape features, which were
#' observed in various co-crystal structures playing an important role in
#' specific protein-DNA binding. The core prediction algorithm enables
#' ultra-fast, high-throughput predictions of shape features for thousands of
#' genomic sequences and is implemented in C++. Since it is likely that
#' features describing additional structural properties or equivalent features
#' derived from different experimental or computational sources will become
#' available, the package has a flexible modular design that easily allows
#' future expansions.
#' In the latest version, we further added additional 9 DNA shape features
#' beyond our previous set of 4 features, and expanded our available repertoire
#' to a total of 13 features, including 6 inter-base pair or base pair-step
#' parameters (HelT, Rise, Roll, Shift, Slide, and Tilt), 6 intra-base pair or
#' base pair-step parameters (Buckle, Opening, ProT, Shear, Stagger,
#' and Stretch), and MGW.
#'
#' Predict biophysical feature
#'
#' Our previous work explained protein-DNA binding specificity based on
#' correlations between MGW and electrostatic potential (EP) observed in
#' experimentally available structures (Joshi, et al., 2007). However, A/T
#' and C/G base pairs carry different partial charge distributions in the
#' minor groove (due primarily to the guanine amino group), which will affect
#' minor-groove EP. We developed a high-throughput method to predict
#' minor-groove EP based on data mining of results from solving the nonlinear
#' Poisson-Boltzmann calculations (Honig & Nicholls, 1995) on 2,297 DNA
#' structures derived from Monte Carlo simulations. DNAshapeR includes EP
#' as an additional feature.
#'
#'
#' @usage getShape(filename, shapeType = 'All', parse = TRUE)
#'
#' @param filename The Name of the input fasta format file, including
#' full path to file if it is located outside the current working directory.
#' @param shapeType A character indicating the shape parameters which can be
#' "MGW", "ProT", "Roll", "HelT" or "All" (meaning all four shapes)
#' @param parse A logical value indicating whether parse the prediction
#' result
#'
#' @return shapeList A List containing shapre prediction result
#'
#' @author Federico Comoglio & Tsu-Pei Chiu
#'
#' @keywords core
#'
#' @examples
#' fn <- system.file("extdata", "CGRsample.fa", package = "DNAshapeR")
#' pred <- getShape(fn)
#' @export getShape

getShape <- function(filename, shapeType = 'Default', parse = TRUE,
                        methylate = FALSE, methylatedPosFile = NULL) {

    # without methylation
    if( methylate == FALSE ){
        defaultOpts <- c( 'MGW', 'HelT', 'ProT', 'Roll', 'EP')
        additionalOpts <- c('Stretch', 'Tilt', 'Buckle', 'Shear', 'Opening',
                            'Rise', 'Shift', 'Stagger', 'Slide')
        stopifnot( shapeType %in% c( defaultOpts, additionalOpts, 'Default' ) )

        if( length(shapeType) == 1 && shapeType == 'Default' ) {
            lapply(defaultOpts, getDNAShape, fastaFilePath = filename)

        } else {
            lapply(shapeType, getDNAShape, fastaFilePath = filename)
        }

        if( parse ) {
            message( 'Parsing files......' )
            if( length(shapeType) == 1 && shapeType == 'Default' ) {
                ln <- paste0( filename, '.', defaultOpts )
                shapeList <- lapply( ln, readShape )
                names( shapeList ) <- defaultOpts

            } else {
                ln <- paste0( filename, '.', shapeType )
                shapeList <- lapply( ln, readShape )
                names( shapeList ) <- shapeType
            }

            message( 'Done' )
            return( shapeList )
        }

    # with methylation
    } else {
        defaultOpts <- c( 'MGW', 'HelT', 'ProT', 'Roll')
        stopifnot( shapeType %in% c( defaultOpts, 'Default' ) )

        # file format converting (Satya)
        convertFileName <- convertMethFile ( filename, methylatedPosFile )


        if( length(shapeType) == 1 && shapeType == 'Default' ) {
            lapply(defaultOpts, getDNAShape, fastaFilePath = convertFileName)

        } else {
            lapply(shapeType, getDNAShape, fastaFilePath = convertFileName)
        }

        if( parse ) {
            message( 'Parsing files......' )
            if( length(shapeType) == 1 && shapeType == 'Default' ) {
                ln <- paste0( convertFileName, '.', defaultOpts )
                shapeList <- lapply( ln, readShape )
                names( shapeList ) <- defaultOpts

            } else {
                ln <- paste0( convertFileName, '.', shapeType )
                shapeList <- lapply( ln, readShape )
                names( shapeList ) <- shapeType
            }

            message( 'Done' )
            return( shapeList )
        }
    }
}


#' Read (parse) DNA shape predictions
#'
#' Read DNA shape predictions
#'
#' @usage readShape(filename)
#'
#' @param filename character name of the file containing shape predictions, including
#' full path to file if it is located outside the current working directory.
#'
#' @return shapeMatrix matrix containing the shape prediction result
#'
#' @author Federico Comoglio & Tsu-Pei Chiu
#'
#' @keywords core
#'
#' @examples
#' fn <- system.file("extdata", "CGRsample.fa", package = "DNAshapeR")
#' pred <- readShape(fn)
#' @export readShape

readShape <- function( filename ) {

    # read file and parse
    records <- scan( filename, what = 'character' )
    recordStart <- grep( '>', records )

    if( length( recordStart ) > 1 ) { #multiple records
        tmp <- paste( records[ ( recordStart[ 1 ] + 1) :
            (recordStart[ 2 ] - 1) ],
                collapse = ',')
    } else { #single record
        tmp <- paste( records[ ( recordStart[ 1 ] + 1) :
            length(records) ],
                collapse = ',')
    }

    expLen <- length( strsplit(tmp, ',')[[1]] )
    message( 'Record length: ', expLen)

    if( length( recordStart ) > 1 ) { #multiple records
        diffrs <- diff( recordStart )
        d <- c( diffrs, diffrs[ 1 ] )
        indicator <- rep( 1 : length( recordStart ), times = d )
    } else { #single record
        indicator <- 1
    }

    records <- split( records, indicator )
    records <- lapply( records, function( x ) x[-1])
    suppressWarnings( records <-  lapply( records,
            function(x) as.numeric( unlist( strsplit( x, ',' ) ) ) ) )
    remove <- which( sapply(records, length) < expLen )
    if( length( remove ) > 0)
        records <- records[ -remove ]
    shapeMatrix <- do.call( 'rbind', records )
    return( shapeMatrix )
}


#' Convert fasta file to methylated file format
#'
#' @usage convertMethFile(fastaFileName, methPositionFileName)
#'
#' @param fastaFileName The name of the input fasta format file, including
#' full path to file if it is located outside the current working directory.
#' @param methPositionFileName The name of the input position file
#' indicating the methlation position
#'
#' @return methFileName fasta file containing methylated Cytosine
#'
#' @author Satyanarayan Rao & Tsu-Pei Chiu

convertMethFile <- function( fastaFileName, methPositionFileName = NULL ) {

    #convertFileName <- fastaFileName
    # read the sequence fasta file first
    fastaFile <- readDNAStringSet( fastaFileName )
    seqName = names( fastaFile )
    seq = paste( fastaFile )
    dfFastaFile <- data.frame( seqName, seq )

    dfFastaFile$seq_name <- as.character ( dfFastaFile$seqName )
    dfFastaFile$seq <- as.character ( dfFastaFile$seq )
    rownames( dfFastaFile ) <- dfFastaFile[["seqName"]]

    # prepare the output file name
    filenameWoExt <- tools::file_path_sans_ext( fastaFileName,
                                                compression = FALSE )
    extension <- tools::file_ext( fastaFileName )
    methFileName <- paste0( filenameWoExt, "_", "methylated" )
    methFileName <- paste( methFileName, extension, sep="." )

    # according to the methPositionFileName convert the methylated nucleotide
    # to MG/Mg and then replace the letter with MQ
    idxM <- which( grepl( "M", dfFastaFile[["seq"]] ) == TRUE )
    idxWoM <- which( grepl("M", dfFastaFile[["seq"]] ) == FALSE )
    seqNum <- dim(dfFastaFile)[1]

    arrayOfNumbers = 1:seqNum
    for ( i in idxM ) {
        dfFastaFile[i, "seq"] = gsub( "MG", "MQ", dfFastaFile[i, "seq"] )
    }

    if (is.null (methPositionFileName)) {
    # no changes to be done at any position
    # check if the fasta file contains sequences with letter "M", or "g".
    # the target is to return the a filename containing sequences with
    # only captial letter and "g" replaced by "Q".
    # Checking if the input fasta sequences have letter M present in them.
    # If a sequence has M letter then the conversion will not be done for
    # that sequence, otherwise all the CpG positions in that sequence
    # will be checking what all sequences have M
        for ( i in idxWoM ){
            dfFastaFile[i, "sequence"] = gsub ("CG", "MQ",
                                               dfFastaFile[i, "seq"])
        }
        OutputSeqs = Biostrings::BStringSet(dfFastaFile[["seq"]])
        names (OutputSeqs) = dfFastaFile[["seqName"]]
        Biostrings::writeXStringSet(OutputSeqs, methFileName)

    } else {
    # mark the methlation according to methPositionFileName
        df.CpGtoMpQ = readNonStandardFastaFile (methPositionFileName)
        names (df.CpGtoMpQ) = c ("seqName", "positions_to_convert_CpG_MpQ")
        rownames (df.CpGtoMpQ) = df.CpGtoMpQ[["seqName"]]
        for (name in df.CpGtoMpQ[["seqName"]]){
            positions_to_convert = df.CpGtoMpQ [name, "positions_to_convert_CpG_MpQ"]
            positions_to_convert = strtoi (unlist (strsplit (positions_to_convert, ",")))
            targetSequence = dfFastaFile[name, "seq"]
            targetSequence = unlist (strsplit(targetSequence, split = ""))
            lengthOftagetSequence = length (targetSequence)
            for (j in positions_to_convert) {
                if (j <=lengthOftagetSequence && targetSequence[j] == "C") {
                    targetSequence[j] = "M"
                    if (j + 1 <= lengthOftagetSequence) {
                        targetSequence[j+1] = "Q"
                    }
                }else {
                    msg = paste ("ERROR:", "position mentioned in",
                                methPositionFileName,
                                "for sequence",
                                name,
                                "is not valid. Letter 'C' was not found at position",
                                j, sep = " ")
                    message (msg)
                    stop ()
                }
            }
            targetSequence = paste0(targetSequence, collapse = "")
            dfFastaFile[name, "seq"] = targetSequence
        }
        # seqinr::write.fasta(sequences = as.list(dfFastaFile[["sequence"]]),
        #                     names = as.list (dfFastaFile[["seq_name"]]),
        #                     file.out = methFileName)
        OutputSequences = Biostrings::BStringSet(dfFastaFile[["seq"]])
        names (OutputSequences) = dfFastaFile[["seqName"]]
        Biostrings::writeXStringSet(OutputSequences, methFileName)
    }
    convertFileName <- methFileName
    return (convertFileName)
}


#' Read the position fasta file
#'
#' @usage readNonStandardFastaFile(filename)
#'
#' @param filename The name of the input position file
#' indicating the methlation position
#'
#' @return df dataframe
#'
#' @author Satyanarayan Rao & Tsu-Pei Chiu
#'

readNonStandardFastaFile <- function( filename ) {
    keys = c()
    content = c ()
    con = file (filename, "r")
    valueKey = ""
    while (TRUE) {
        line = gsub ("\n", "", readLines( con, n = 1 ))

        if ( length( line ) == 0 ){
            break
        }
        if ( line == "" ) {
            next
        } else if( ">" == substring( line, 1, 1 ) ) {
            key = substring( line, 2 )

            while( TRUE ){
                tmpKey = gsub( "\n","",readLines(con, n = 1 ) )
                if (length( tmpKey ) == 0 ){
                    break
                }

                if ( tmpKey == ""){
                    next

                } else if( ">" == substring( tmpKey, 1, 1 ) ) {
                    content <- c( content, valueKey )
                    keys <- c( keys, key )
                    valueKey <- ""
                    key <- substring(tmpKey ,2)

                } else {
                    valueKey <- paste0( valueKey, tmpKey,
                                 collapse="" )
                }
            }
            content <- c( content, valueKey )
            keys <- c( keys, key )
        } else {
            key <- "1"
            valueKey <- line

            while( TRUE ) {
                line <- readLines( con, n = 1 )
                if ( length(line) == 0 ){
                    break
                }
                valueKey <- paste0(valueKey, line, collapse="")
            }
            content <- c (line_to_append)
            keys <- c (key)
        }
    }
    close (con)

    df <- data.frame (keys = keys, content = content)
    df$keys <- as.character(df$keys)
    df$content <- as.character(df$content)
    return (df)
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# DNAshapeR Unit testing
# 2015 
# Tsu-Pei Chiu, Rohs Lab, USC
# Federico Comoglio, Green lab, CIMR
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

fn <- system.file( 'extdata', 'SingleSeqsample.fa.MGW', package = 'DNAshapeR' )
mgw <- readShape(fn)
expect_equal( length(mgw), 24 )

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

getDNAShape <- function(fastaFilePath, shapeType) {
    .Call('_DNAshapeR_getDNAShape', PACKAGE = 'DNAshapeR', fastaFilePath, shapeType)
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# DNAshapeR
# 2015,2016 
# Tsu-Pei Chiu, Rohs Lab, USC
# Federico Comoglio, Green lab, CIMR
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#' DNAshapeR package for high-throughput prediction of DNA shape features
#' 
#' The main functions in the package are \code{\link{getFasta}} and
#' \code{\link{getShape}}. Shape predictions can be additionally plotted with \code{\link{plotShape}}.
#' See package vignette for examples.
#'
#' All support questions should be posted to the Bioconductor
#' support site: \url{support.bioconductor.org}, using DNAshapeR as a tag.
#'
#' @references
#'
#' DNAshapeR reference:
#' 
#' T.-P. Chiu*, F. Comoglio*, T. Zhou, L. Yang, R. Paro, and R. Rohs: DNAshapeR: an R/Bioconductor package for DNA shape prediction and feature encoding (2016). Bioinformatics \url{http://bioinformatics.oxfordjournals.org/content/early/2016/01/09/bioinformatics.btv735} (*equal contributor in alphabetic order)
#'
#' @author Tsu-Pei Chiu and Federico Comoglio
#' 
#' @docType package
#' @name DNAshapeR-package
#' @aliases DNAshapeR-package
#' @keywords package
NULL

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# DNAshapeR
# 2015
# Tsu-Pei Chiu, Rohs Lab, USC
# Federico Comoglio, Green lab, CIMR
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#' Encode k-mer DNA sequence and n-th order DNA Shape features
#'
#' DNAshapeR can be used to generate feature vectors for a user-defined model.
#' These models can be based on DNA sequence (1-mer, 2-mer, 3-mer) or DNA
#' shape (MGW, Roll, ProT, HelT) features or any combination thereof. Sequence
#' is encoded as four binary features (i.e., 0001 for adenine, 0010 for
#' cytosine, 0100 for guanine, and 1000 for thymine, for encoding of 1-mers)
#' at each nucleotide position (Zhou, et al., 2015). Encoding of 2-mers and
#' 3-mers (16 and 64 binary features at each position, respectively) is also
#' supported. Shape features include first and second order (or higher order)
#' values for the four structural parameters MGW, Roll, ProT and HelT. The
#' second order shape features are product terms of values for the same
#' category of shape features at adjacent positions. The function allows to
#' generate any subset of these features, e.g. a given shape category or first
#' order shape features, and any desired combination of shape and sequence
#' features. Feature encoding returns a feature matrix for a dataset of
#' multiple sequences, in which each sequence generates a concatenated feature
#' vector. The output of this function can be used directly for any statistical
#' machine learning method.
#'
#'
#' @usage encodeSeqShape(fastaFileName, shapeMatrix, featureNames, normalize)
#'
#' @param fastaFileName A character name of the input fasta format file,
#' including full path to file if it is located outside the current working
#' directory.
#' @param shapeMatrix A matrix containing DNAshape prediction result
#' @param featureNames A vector containing a combination of user-defined
#' sequence and shape parameters. The parameters can be any combination of
#' "k-mer", "n-shape", "n-MGW", "n-ProT", "n-Roll", "n-HelT" (k, n are
#' integers)
#' @param normalize A logical indicating whether to perform
#' normalization. Default to TRUE.
#' @return featureVector A matrix containing encoded features. Sequence
#' features are represented as binary numbers, while shape features are
#' represented as real numbers.
#'
#' @author Tsu-Pei Chiu
#'
#' @keywords core
#'
#' @examples
#' fn <- system.file("extdata", "CGRsample_short.fa", package = "DNAshapeR")
#' pred <- getShape(fn)
#' featureNames <- c("1-shape")
#' featureVector <- encodeSeqShape(fn, pred, featureNames)
#' @export encodeSeqShape

encodeSeqShape <- function( fastaFileName, shapeMatrix, featureNames, normalize = TRUE ) {
    ds <- readDNAStringSet(fastaFileName, "fasta")

    featureVector <- c()
    n <- length( featureNames )

    for( i in seq_len(n) ){
        featureName <- unlist(strsplit(featureNames[i], "-"))
        switch( featureName[2],
            mer = { featureVector <- cbind( featureVector,
                    encodeKMerSeq( as.numeric( featureName[1] ), ds ) ) },
            Hbond = { featureVector <- cbind( featureVector,
                    encodeKmerHbond( as.numeric( featureName[1] ), ds ) ) },
            shape = {
                    featureVector <- cbind( featureVector,
                        normalizeShape(
                            encodeNstOrderShape( as.numeric( featureName[1] ),
                            shapeMatrix$MGW, "MGW" ), as.numeric( featureName[1] ),
                            "MGW" , normalize ),

                        normalizeShape(
                            encodeNstOrderShape( as.numeric( featureName[1] ),
                            shapeMatrix$ProT, "ProT" ), as.numeric( featureName[1] ),
                            "ProT" , normalize ),

                        normalizeShape(
                            encodeNstOrderShape( as.numeric( featureName[1] ),
                            shapeMatrix$Roll, "Roll" ), as.numeric( featureName[1] ),
                            "Roll" , normalize ),

                        normalizeShape(
                            encodeNstOrderShape( as.numeric( featureName[1] ),
                            shapeMatrix$HelT, "HelT" ), as.numeric( featureName[1] ),
                            "HelT" , normalize )
                    )
            },
            {
                featureVector <- cbind( featureVector,
                    normalizeShape(
                        encodeNstOrderShape( as.numeric( featureName[1] ),
                        shapeMatrix[[featureName[2]]], featureName[2] ), as.numeric( featureName[1] ),
                        featureName[2] , normalize) )
            }
        )
    }

    return (featureVector)
}


#' Encode k-mer DNA sequence features
#'
#' DNAshapeR can be used to generate feature vectors for a user-defined model.
#' The model can be a k-mer sequence. Sequence is encoded in four binary
#' features (i.e., in terms of 1-mers, 0001 for adenine, 0010 for cytosine,
#' 0100 for guanine, and 1000 for thymine) at each nucleotide position
#' (Zhou, et al., 2015). The function permits an encoding of 2-mers and 3-mers
#' (16 and 64 binary features at each position, respectively).
#'
#' @usage encodeKMerSeq(k, dnaStringSet)
#'
#' @param k A number indicating k-mer sequence encoding
#' @param dnaStringSet A DNAStringSet object of the inputted fasta file
#' @return featureVector A matrix containing encoded features. Sequence
#' feature is represented as binary numbers
#' @author Tsu-Pei Chiu

encodeKMerSeq <- function( k, dnaStringSet ){
    # create a lookup table
    lookupTable <- diag( 4**k )
    row.names( lookupTable ) <- mkAllStrings( c( "A", "C", "G", "T" ), k )

    # pre-allocate the featureVector
    l <- nchar( toString( dnaStringSet[1] ) ) # get sequence length
    n <- ( l - k + 1 ) * ( 4 ** k )
    m <- length( dnaStringSet )
    featureVector <- matrix(0L, m, n)

    for( i in 1 : length( dnaStringSet ) ){
        # encode k-mer feature
        features <- c()
        seq <- toupper( toString( dnaStringSet[i] ) )
        for ( j in 1 : ( nchar( seq )-k+1) ){

            if( is.na( match( substr( seq, j, j+k-1),
                    row.names( lookupTable ) ) ) ){
                # do nothing

            }else{
                featureVector[i, ((j-1)*(4**k)+1):(j*(4**k))] <-
                    lookupTable[ substr( seq, j, j+k-1), ]
            }
        }
    }
    row.names( featureVector ) <- names( dnaStringSet )

    return ( featureVector )
}


#' Encode n-st order shape features
#
#' DNAshapeR can be used to generate feature vectors for a user-defined model.
#' The model can be a shape model. There are four structural parameters
#' including MGW, Roll, ProT and HelT. The second order shape features are
#' product terms of values for the same category of shape features at adjacent
#' positions.
#'
#' @usage encodeNstOrderShape(n, shapeMatrix, shapeType)
#'
#' @param n A number indicating n-st order shape encoding
#' @param shapeMatrix A matrix containing DNAshape prediction result
#' @param shapeType A character name of shape (MGW, Roll, ProT, HelT) features
#' @return featureVector A matrix containing encoded features. shape feature is
#' represented as continuous numbers
#' @author Tsu-Pei Chiu

encodeNstOrderShape <- function( n, shapeMatrix, shapeType ){
    # assign average value to NA
    #shapeMatrix[ is.na( shapeMatrix ) ] <- 0
    switch( shapeType,
        MGW = { shapeMatrix[ is.na( shapeMatrix ) ] <- 5.072 },
        ProT = { shapeMatrix[ is.na( shapeMatrix ) ] <- -6.792 },
        Roll = { shapeMatrix[ is.na( shapeMatrix ) ] <- -0.698 },
        HelT = { shapeMatrix[ is.na( shapeMatrix ) ] <- 34.326 },
        EP = { shapeMatrix[ is.na( shapeMatrix ) ] <- -6.505 },

        Stretch = { shapeMatrix[ is.na( shapeMatrix ) ] <- -0.028 },
        Tilt = { shapeMatrix[ is.na( shapeMatrix ) ] <- -0.008 },
        Buckle = { shapeMatrix[ is.na( shapeMatrix ) ] <- 0.097 },
        Shear = { shapeMatrix[ is.na( shapeMatrix ) ] <- -0.009 },
        Opening = { shapeMatrix[ is.na( shapeMatrix ) ] <- -0.24 },
        Rise = { shapeMatrix[ is.na( shapeMatrix ) ] <- 3.342 },
        Shift = { shapeMatrix[ is.na( shapeMatrix ) ] <- 0.008 },
        Stagger = { shapeMatrix[ is.na( shapeMatrix ) ] <- -0.013 },
        Slide = { shapeMatrix[ is.na( shapeMatrix ) ] <- -1.526 }
    )


    singleSeq <- FALSE
    if( nrow(shapeMatrix)[1] == 1 )
        singleSeq <- TRUE

    # trim both 2 bps end
    if( shapeType == "MGW" || shapeType == "ProT" || shapeType == "EP" ||
        shapeType == "Stretch" || shapeType == "Buckle" ||
        shapeType == "Shear" || shapeType == "Opening" ||
        shapeType == "Stagger" ){
            shapeMatrix <- shapeMatrix[, -c(1, 2, ncol( shapeMatrix )-1,
                ncol( shapeMatrix ))]

    }else if( shapeType == "Roll" || shapeType == "HelT" ||
            shapeType == "Tilt" || shapeType == "Rise" ||
            shapeType == "Shift" || shapeType == "Slide" ){
      shapeMatrix <- shapeMatrix[, -c(1, ncol( shapeMatrix ))]
    }

    if(singleSeq)
        shapeMatrix <- t(shapeMatrix)

    # encode k-st feature
    featureVector <- NULL
    if( n == 1 ){
        featureVector = shapeMatrix

    }else{
        m <- ncol( shapeMatrix )
        # normalization
        shapeMatrix <- normalizeShape( featureVector = shapeMatrix,
                                       thOrder = 1, shapeType = shapeType,
                                       normalize = TRUE )

        for ( i in 1 : ( m-n+1 )){
            feature <- shapeMatrix[, i]
            for ( j in ( i+1 ) : ( i+n-1 ) )
                feature <- feature * shapeMatrix[, j]

            featureVector <- cbind( featureVector, unlist( feature ) )
        }
    }

    return ( featureVector )
}


#' Normalize n-st order shape features
#'
#' @usage normalizeShape(featureVector, thOrder, shapeType, normalize)
#'
#' @param featureVector A matrix containing encoded features.
#' @param thOrder A number indicating n-st order shape encoding
#' @param shapeType A character name of shape (MGW, Roll, ProT, HelT) features
#' @param normalize A logical indicating whether to perform
#' normalization. Default to TRUE.
#' @return featureVector A matrix containing encoded features.
#' @author Tsu-Pei Chiu

normalizeShape <- function( featureVector, thOrder, shapeType, normalize ){
    tableName <- c("MGW", "ProT", "Roll", "HelT", "EP", "Stretch", "Tilt", "Buckle",
                    "Shear", "Opening", "Rise", "Shift", "Stagger", "Slide")
    minTable <- c( 2.85, -16.51, -8.57, 30.94, -13.59, -0.05, -3.02, -9.26,
                    -0.3, -3.71, 3.05, -0.42, -0.42, -1.95 )
    maxTable <- c( 6.2, -0.03, 8.64, 38.05, -4.47, 0.04, 5.4, 7.66, 0.28, 1.06,
                    3.74, 0.51, 0.2, -0.97 )
    names( minTable ) <- tableName
    names( maxTable ) <- tableName

    if ( normalize  ){
        if( thOrder == 1){
            featureVector <- normalize( featureVector, maxTable[shapeType],
                                        minTable[shapeType] )

        }else{
            featureVector <- normalize( featureVector, max(featureVector),
                                        min(featureVector) )
        }
    }

    return ( featureVector )
}


#' Min-Max normalization
#'
#' @usage normalize(x, max, min)
#'
#' @param x A matrix containing encoded features
#' @param max A number maximum number for Min-Max Normalization
#' @param min A number minimum number for Min-Max Normalization
#' @return  featureVector A matrix containing encoded features. shape feature is
#' represented as continuous numbers
#' @author Tsu-Pei Chiu

normalize <- function( x, max, min ){
  return ( (x-min)/(max-min) )
}


#' encode Hbond
#'
#' @usage encodeKmerHbond (filepath)
#' @param k k-mer sequence
#' @param dnaStringSet
#' @return featureVector

encodeKmerHbond <- function ( k, dnaStringSet ){
  # create a lookup table
  k <- 1
  lookupTable <- rbind( c(1,0,0,0,0,1,0,0,1,0,0,0,0,0,0,1), c(0,0,1,0,0,1,0,0,1,0,0,0,1,0,0,0),
                        c(1,0,0,0,1,0,0,0,0,1,0,0,0,0,1,0), c(0,0,0,1,1,0,0,0,0,1,0,0,1,0,0,0) )
  row.names( lookupTable ) <- mkAllStrings( c( "A", "C", "G", "T" ), k )

  # pre-allocate the featureVector
  #l <- nchar( toString( dnaStringSet[1] ) ) # get sequence length
  #n <- ( l - k + 1 ) * ( 16 ** k )
  #m <- length( dnaStringSet )
  #featureVector <- matrix(0L, m, n)
  featureVector <- c()

  for( j in 1 : length( dnaStringSet ) ){
    # encode k-mer feature
    features <- c()
    seq <- toString( dnaStringSet[j] )
    for ( j in 1 : ( nchar( seq )-k+1) ){
      if( is.na( match( substr( toupper( seq ), j, j+k-1), row.names( lookupTable ) ) ) ){
        features <- c( features, rep( 0, 16**k ) )

      }else{
        features <- c( features, lookupTable[ substr( toupper( seq ), j, j+k-1), ] )
      }
    }

    featureVector <- rbind( featureVector, features )
  }
  row.names( featureVector ) <- names( dnaStringSet )

  return ( featureVector )

}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# DNAshapeR
# 2015
# Tsu-Pei Chiu, Rohs Lab, USC
# Federico Comoglio, Green lab, CIMR
#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#' Extract fasta sequence given a set of genomic intervals and a reference
#' genome.
#'
#' DNAshapeR can predict DNA shape features from custom FASTA files or directly
#' from genomic coordinates in the form of a GRanges object within BioConductor
#' (see
#' <https://bioconductor.org/packages/release/bioc/html/GenomicRanges.html>
#' for more information).
#'
#' @usage getFasta(GR, BSgenome, width = 1e3, filename = 'tmp.fa')
#'
#' @param GR A GRanges object indicating genomic coordinates
#' @param BSgenome A BSgenome object indicating the genome of interest
#' @param width A number indicating a fixed width of sequences
#' @param filename The Name of the input fasta format file, including
#' full path to file if it is located outside the current working directory
#' @return writes a fasta file
#'
#' @author Federico Comoglio
#'
#' @keywords core
#'
#' @examples
#' gr <- GRanges(seqnames = c("chrI"),
#' strand = c("+", "-", "+"),
#' ranges = IRanges(start = c(100, 200, 300), width = 100))
#' library(BSgenome.Scerevisiae.UCSC.sacCer3)
#' getFasta(gr, BSgenome = Scerevisiae, width = 100, filename = "tmp.fa")
#' fn <- "tmp.fa"
#' pred <- getShape(fn)
#' @export getFasta

getFasta <- function( GR, BSgenome, width = 1e3, filename = 'tmp.fa' ) {
    GR <- resize( GR, width = width, fix = 'center' )
    seqlengths <- NULL
    `seqlengths<-`<- NULL
    seqlengths( GR ) <- seqlengths( BSgenome )[ names( seqlengths( GR ) ) ]
    GR <- trim( GR )

    idx <- which( width( GR ) < width )
    if( length( idx ) > 0 )
        GR <- GR[ -idx ]
    message( 'Removed sequences #: ', list( idx ))

    sequences <- getSeq( BSgenome, names = GR )
    names( sequences ) <- paste0( 'seq', 1 : length(GR) )
    writeXStringSet(sequences, filepath = filename, format = "fasta")
}

---
title: "Bioconductor Package Vignette"
author: "Tsu-Pei Chiu, Federico Comoglio and Remo Rohs"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
    %\VignetteIndexEntry{DNAshapeR}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

## Introduction

DNAshapeR predicts DNA shape features in an ultra-fast, high-throughput 
manner from genomic sequencing data. The package takes either nucleotide 
sequence or genomic intervals as input, and generates various graphical 
representations for further analysis. DNAshapeR further encodes DNA 
sequence and shape features for statistical learning applications by 
concatenating feature matrices with user-defined combinations of k-mer 
and DNA shape features that can be readily used as input for machine 
learning algorithms.

In this vignette, you will learn: 

* how to load/install DNAshapeR

* how to predict DNA shape features

* how to visualize DNA shape predictions

* how to encode sequence and shape features, and apply them


## Load DNAshapeR

```{r eval=TRUE}
library(DNAshapeR)
```

## Predict DNA shape features
The core of DNAshapeR, the DNAshape method (Zhou, et al., 2013), 
uses a sliding pentamer window where structural features unique 
to each of the 512 distinct pentamers define a vector of minor 
groove width (MGW), Roll, propeller twist (ProT), and helix twist 
(HelT) at each nucleotide position. MGW and ProT define base-pair 
parameters whereas Roll and HelT represent base pair-step parameters. 
The values for each DNA shape feature as function of its pentamer 
sequence were derived from all-atom Monte Carlo simulations where DNA 
structure is sampled in collective and internal degrees of freedom in 
combination with explicit sodium counter ions (Zhang, et al., 2014). 
The Monte Carlo simulations were analyzed with a modified Curves 
approach (Zhou, et al., 2013). Average values of each shape feature 
for each pentamer were derived from analyzing the ensemble of Monte 
Carlo predictions for 2,121 DNA fragments of 12−27 base pairs in length. 
DNAshapeR predicts the four DNA shape features MGW, HelT, ProT, and Roll, 
which were observed in various cocrystal structures as playing an important 
role in specific protein-DNA binding. 

In the latest version, we further added additional 9 DNA shape features 
beyond our previous set of 4 features, and expanded our available 
repertoire to a total of 13 features, including 6 inter-base pair or 
base pair-step parameters (HelT, Rise, Roll, Shift, Slide, and Tilt), 
6 intra-base pair or base pair-step parameters (Buckle, Opening, ProT, 
Shear, Stagger, and Stretch), and MGW.


## Predict biophysical feature
Our previous work explained protein-DNA binding specificity based on 
correlations between MGW and (electrostatic potential) EP observed in 
experimentally available structures (Joshi, et al., 2007). However, A/T 
and C/G base pairs carry different partial charge distributions in the 
minor groove (due primarily to the guanine amino group), which will affect 
minor-groove EP. We developed a high-throughput method, named DNAphi, to 
predict minor-groove EP based on data mining of results from solving the 
nonlinear Poisson-Boltzmann calculations (Honig & Nicholls, 1995) on 2,297 
DNA structures derived from Monte Carlo simulations. DNAshapeR includes EP 
as an additional feature.


## Predict DNA shape feature due to CpG methylation
To achieve a better mechanistic understanding of the effect of CpG 
methylation on local DNA structure, we developed a high-throughput 
method, named methyl-DNAshape, for predicting the impact of cytosine 
methylation on DNA shape features. In analogy to the DNAshape method 
(Zhou, et al., 2013), the method predicts DNA shape features (ProT, 
HelT, Roll, and MGW) in the context of CpG methylation based on 
methyl-DNAshape Pentamer Query Table (mPQT) derived from the results 
of all-atom Monte Carlo simulations on a total of 3,518 DNA fragments 
of lengths varying from 13 to 24 bp.  



DNAshapeR can predict DNA shape features from custom FASTA files or directly 
from genomic coordinates in the form of a GRanges object within Bioconductor 
(see <https://bioconductor.org/packages/release/bioc/html/GenomicRanges.html> 
for more information). 

### From FASTA file

To predict DNA shape features from a FASTA file

```{r eval=TRUE}
library(DNAshapeR)
fn <- system.file("extdata", "CGRsample.fa", package = "DNAshapeR")
pred <- getShape(fn)
```

### From genomic intervals (e.g. TFs binding sites, CpG islands, replication origins, ...)

To predict DNA shape from genomic intervals stored as GRanges object, a 
reference genome is required. Several reference genomes are available 
within BioConductor as BSgenome objects 
(see <http://bioconductor.org/packages/release/bioc/html/BSgenome.html> 
for more information). For example, the sacCer3 release of the *S.Cerevisiae* 
genome can be retrieved. Given a reference genome, the **getFasta** function 
first extracts the DNA 
sequences based on the provided genomic coordinates, and then performs shape 
predictions within a user-defined window (of size equal to width, 100 bp in 
the example below) computed from the center of each genomic interval:

```{r eval=FALSE}
# Install Bioconductor packages
if (!requireNamespace("BiocManager", quietly=TRUE))
    install.packages("BiocManager")
BiocManager::install("BSgenome.Scerevisiae.UCSC.sacCer3")

library(BSgenome.Scerevisiae.UCSC.sacCer3)

# Create a query GRanges object
gr <- GRanges(seqnames = c("chrI"),
            strand = c("+", "-", "+"),
            ranges = IRanges(start = c(100, 200, 300), width = 100))
getFasta(gr, Scerevisiae, width = 100, filename = "tmp.fa")
fn <- "tmp.fa"
pred <- getShape(fn)
```

### From public domain projects

The genomic intervals can also be obtained from public domain projects, 
including ENCODE, NCBI, Ensembl, etc. The AnnotationHub package 
(see <http://bioconductor.org/packages/release/bioc/html/AnnotationHub.html> 
for more information) provides an interface to retrieve genomic intervals 
from these multiple online project resources.The genomic intervals of interest
can be selected progressively through the functions of **sebset** and **query** 
with keywords, and can be subjected as 
an input of GRanges object to **getFasta** function.

```{r eval=FALSE}
# Install Bioconductor packages
library(BSgenome.Hsapiens.UCSC.hg19)
library(AnnotationHub)

ah <- AnnotationHub()
ah <- subset(ah, species=="Homo sapiens")
ah <- query(ah, c("H3K4me3", "Gm12878", "Roadmap"))
getFasta(ah[[1]], Hsapiens, width = 150, filename = "tmp.fa")
fn <- "tmp.fa"
pred <- getShape(fn)
```


### From FASTA file with methylated DNA sequence

To predict DNA shape features in the context of CpG methylation, one 
can prepare a FASTA file of sequence with symbol ‘Mg’: ‘M’ referring 
to cytosine of methylated CpG on the leading strand and ‘g’ referring 
the cytosine of methylated CpG on lagging strand. For example,

\>seq1

GTGTCACMgCGTCTATACG

notifying the cytosine at position 8th on the leading strand and the 
one at position 9th on the lagging strand are methylated.

```{r eval=TRUE}
library(DNAshapeR)
fn_methy <- system.file("extdata", "MethylSample.fa", package = "DNAshapeR")
pred_methy <- getShape(fn_methy, methylate = TRUE)
pred_methy$MGW
```


### From FASTA and methylated position files

To predict DNA shape features in the context of CpG methylation, 
in addition to providing regular FASTA file (without symbolizing ‘Mg’) 
one can provide an additional input file identifying methylated positions. 
For example, 

\>seq1

4,16

notifying the cytosine at position 4th and 16th on leading strand, and 5th 
and 17th on lagging strand are methylated.


```{r eval=TRUE}
library(DNAshapeR)
fn_methy <- system.file("extdata", "SingleSeqsample.fa", package = "DNAshapeR")
fn_methy_pos <- system.file("extdata", "MethylSamplePos.fa", package = "DNAshapeR")
pred_methy <- getShape(fn_methy, methylate = TRUE, methylatedPosFile = fn_methy_pos)
pred_methy$MGW
```


## Visualize DNA shape prediction
DNAshapeR can be used to generate various graphical representations for 
further analyses. The prediction result can be visualized in the form of 
scatter plots (Comoglio, et al., 2015), heat maps (Yang, et al., 2014), 
or genome browser tracks (Chiu, et al., 2015).

### Ensemble representation: metashape plot
The prediction result can be visualized in the metaprofiles of DNA shape 
features.

```{r fig.width=7, fig.height=7, fig.align='center', eval=TRUE}
plotShape(pred$MGW)
#plotShape(pred$ProT)
#plotShape(pred$Roll)
#plotShape(pred$HelT)
```

### Ensemble representation: heat map
The prediction result can be visualized in the heatmap of DNA shape features.

```{r fig.width=7, fig.height=7, fig.align='center', eval=TRUE}
library(fields)
heatShape(pred$ProT, 20)
#heatShape(pred$MGW, 20)
#heatShape(pred$Roll[1:500, 1:1980], 20)
#heatShape(pred$HelT[1:500, 1:1980], 20)
```


### Individual representation: genome browser-like tracks
The prediction result can be visualized in the form of genome browser tracks.

*Note that the input data should only contain one sequence.

```{r fig.width=7, fig.height=7, fig.align='center', eval=TRUE}
fn2 <- system.file("extdata", "SingleSeqsample.fa", package = "DNAshapeR")
pred2 <- getShape(fn2)
trackShape(fn2, pred2) # Only for single sequence file
```

## Encode sequence and shape features
DNAshapeR can be used to generate feature vectors for a user-defined model. 
These models can consist of either sequence features (1-mer, 2-mer, 3-mer), 
shape features (MGW, Roll, ProT, HelT), or any combination of those two. 
For 1-mer features, sequence is encoded in form of four binary numbers 
(i.e., in terms of 1-mers, 1000 for adenine, 0100 for cytosine, 0010 for 
guanine, and 0001 for thymine) at each nucleotide position (Zhou, et al., 
2015). The encoding function of the DNAshapeR package enables the 
determination of higher order sequence features, for example, 2-mers and 
3-mers (16 and 64 binary features at each position, respectively). The 
user can also choose to include second order shape features in the 
generated feature vector. The second order shape features are product 
terms of values for the same category of shape features (MGW, Roll, ProT or 
HelT) at adjacent positions. The feature encoding function of DNAshapeR 
enables the generation of any subset of these features, either only a 
selected shape category or first order shape features, and any combination 
with shape or sequence features. The result of feature encoding for each 
sequence is a chimera feature vector.

### Encoding process
A feature type vector should be defined before encoding. The vector can be
any combination of characters of “k-mer”, “n-shape”, “n-MGW”, “n-ProT”, 
“n-Roll”, “n-HelT” (k, n are integers) where “1-shape” refers to first 
order and “2-shape” to second order shape features.

```{r eval=TRUE}
library(Biostrings)
fn3 <- system.file("extdata", "PBMsample_short.fa", package = "DNAshapeR")
pred3 <- getShape(fn3)
featureType <- c("1-mer", "1-shape")
featureVector <- encodeSeqShape(fn3, pred3, featureType)
head(featureVector)
```

### Showcase of statistical machine learning application
Feature encoding of multiple sequences thus results in a feature matrix, 
which can be used as input for variety of statistical machine learning 
methods. For example, an application is the quantitative modeling of 
PBM derived protein-DNA binding by linear regression as demonstrated 
below.

First, pre-computed binding affinity values are combined with experimental 
information in a data frame structure.

```{r eval=TRUE}
fn4 <- system.file("extdata", "PBMsample_short.s", package = "DNAshapeR")
experimentalData <- read.table(fn4)
df <- data.frame(affinity=experimentalData$V1, featureVector)
```

Then, a machine learning package (which can be any learning tools) is used 
to train a multiple linear regression (MLR) model based on 3-fold 
cross-validation. In this example, we used the caret package 
(see <http://caret.r-forge.r-project.org/> for more information).

```{r eval=TRUE}
library(caret)

trainControl <- trainControl(method = "cv", number = 3, 
                savePredictions = TRUE)
model <- train (affinity~ ., data = df, 
                trControl=trainControl, method="lm", preProcess=NULL)
model
```


## Session Info
```{r eval=TRUE}
sessionInfo()
```

## References


Chiu, T.P., et al. GBshape: a genome browser database for DNA shape 
annotations. Nucleic Acids Res 2015.

Comoglio, F., et al. High-resolution profiling of Drosophila replication 
start sites reveals a DNA shape and chromatin signature of metazoan origins. 
Cell reports 2015;11(5):821-834.

Yang, L., et al. TFBSshape: a motif database for DNA shape features of 
transcription factor binding sites. Nucleic Acids Res 2014;42
(Database issue):D148-155.

Zhou, T., et al. DNAshape: a method for the high-throughput prediction of
DNA structural features on a genomic scale. Nucleic Acids Res 2013;41
(Web Server issue):W56-62.

If you use DNAshapeR for your work, please cite the following publication:

Chiu, T-P., et al. DNAshapeR: an R/Bioconductor package for DNA shape 
prediction and feature encoding (2016).

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/encodeSeqShape.R
\name{encodeKmerHbond}
\alias{encodeKmerHbond}
\title{encode Hbond}
\usage{
encodeKmerHbond (k, dnaStringSet )
}
\arguments{
\item{k}{k-mer sequence}

\item{dnaStringSet}{dnaStringSet}
}
\value{
featureVector
}
\description{
encode Hbond
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/core.R
\docType{package}
\name{DNAshapeR-package}
\alias{DNAshapeR-package}
\title{DNAshapeR package for high-throughput prediction of DNA shape features}
\description{
The main functions in the package are \code{\link{getFasta}} and
\code{\link{getShape}}. Shape predictions can be additionally plotted with \code{\link{plotShape}}.
See package vignette for examples.
}
\details{
All support questions should be posted to the Bioconductor
support site: \url{support.bioconductor.org}, using DNAshapeR as a tag.
}
\references{
DNAshapeR reference:

T.-P. Chiu*, F. Comoglio*, T. Zhou, L. Yang, R. Paro, and R. Rohs: DNAshapeR: an R/Bioconductor package for DNA shape prediction and feature encoding (2016). Bioinformatics \url{http://bioinformatics.oxfordjournals.org/content/early/2016/01/09/bioinformatics.btv735} (*equal contributor in alphabetic order)
}
\author{
Tsu-Pei Chiu and Federico Comoglio
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/encodeSeqShape.R
\name{encodeSeqShape}
\alias{encodeSeqShape}
\title{Encode k-mer DNA sequence and n-th order DNA Shape features}
\usage{
encodeSeqShape(fastaFileName, shapeMatrix, featureNames, normalize)
}
\arguments{
\item{fastaFileName}{A character name of the input fasta format file,
including full path to file if it is located outside the current working
directory.}

\item{shapeMatrix}{A matrix containing DNAshape prediction result}

\item{featureNames}{A vector containing a combination of user-defined
sequence and shape parameters. The parameters can be any combination of
"k-mer", "n-shape", "n-MGW", "n-ProT", "n-Roll", "n-HelT" (k, n are
integers)}

\item{normalize}{A logical indicating whether to perform
normalization. Default to TRUE.}
}
\value{
featureVector A matrix containing encoded features. Sequence
features are represented as binary numbers, while shape features are
represented as real numbers.
}
\description{
DNAshapeR can be used to generate feature vectors for a user-defined model.
These models can be based on DNA sequence (1-mer, 2-mer, 3-mer) or DNA
shape (MGW, Roll, ProT, HelT) features or any combination thereof. Sequence
is encoded as four binary features (i.e., 0001 for adenine, 0010 for
cytosine, 0100 for guanine, and 1000 for thymine, for encoding of 1-mers)
at each nucleotide position (Zhou, et al., 2015). Encoding of 2-mers and
3-mers (16 and 64 binary features at each position, respectively) is also
supported. Shape features include first and second order (or higher order)
values for the four structural parameters MGW, Roll, ProT and HelT. The
second order shape features are product terms of values for the same
category of shape features at adjacent positions. The function allows to
generate any subset of these features, e.g. a given shape category or first
order shape features, and any desired combination of shape and sequence
features. Feature encoding returns a feature matrix for a dataset of
multiple sequences, in which each sequence generates a concatenated feature
vector. The output of this function can be used directly for any statistical
machine learning method.
}
\examples{
fn <- system.file("extdata", "CGRsample_short.fa", package = "DNAshapeR")
pred <- getShape(fn)
featureNames <- c("1-shape")
featureVector <- encodeSeqShape(fn, pred, featureNames)
}
\author{
Tsu-Pei Chiu
}
\keyword{core}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/encodeSeqShape.R
\name{normalizeShape}
\alias{normalizeShape}
\title{Normalize n-st order shape features}
\usage{
normalizeShape(featureVector, thOrder, shapeType, normalize)
}
\arguments{
\item{featureVector}{A matrix containing encoded features.}

\item{thOrder}{A number indicating n-st order shape encoding}

\item{shapeType}{A character name of shape (MGW, Roll, ProT, HelT) features}

\item{normalize}{A logical indicating whether to perform
normalization. Default to TRUE.}
}
\value{
featureVector A matrix containing encoded features.
}
\description{
Normalize n-st order shape features
}
\author{
Tsu-Pei Chiu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/encodeSeqShape.R
\name{encodeKMerSeq}
\alias{encodeKMerSeq}
\title{Encode k-mer DNA sequence features}
\usage{
encodeKMerSeq(k, dnaStringSet)
}
\arguments{
\item{k}{A number indicating k-mer sequence encoding}

\item{dnaStringSet}{A DNAStringSet object of the inputted fasta file}
}
\value{
featureVector A matrix containing encoded features. Sequence
feature is represented as binary numbers
}
\description{
DNAshapeR can be used to generate feature vectors for a user-defined model.
The model can be a k-mer sequence. Sequence is encoded in four binary
features (i.e., in terms of 1-mers, 0001 for adenine, 0010 for cytosine,
0100 for guanine, and 1000 for thymine) at each nucleotide position
(Zhou, et al., 2015). The function permits an encoding of 2-mers and 3-mers
(16 and 64 binary features at each position, respectively).
}
\author{
Tsu-Pei Chiu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getFasta.R
\name{getFasta}
\alias{getFasta}
\title{Extract fasta sequence given a set of genomic intervals and a reference
genome.}
\usage{
getFasta(GR, BSgenome, width = 1e3, filename = 'tmp.fa')
}
\arguments{
\item{GR}{A GRanges object indicating genomic coordinates}

\item{BSgenome}{A BSgenome object indicating the genome of interest}

\item{width}{A number indicating a fixed width of sequences}

\item{filename}{The Name of the input fasta format file, including
full path to file if it is located outside the current working directory}
}
\value{
writes a fasta file
}
\description{
DNAshapeR can predict DNA shape features from custom FASTA files or directly
from genomic coordinates in the form of a GRanges object within BioConductor
(see
<https://bioconductor.org/packages/release/bioc/html/GenomicRanges.html>
for more information).
}
\examples{
gr <- GRanges(seqnames = c("chrI"),
strand = c("+", "-", "+"),
ranges = IRanges(start = c(100, 200, 300), width = 100))
library(BSgenome.Scerevisiae.UCSC.sacCer3)
getFasta(gr, BSgenome = Scerevisiae, width = 100, filename = "tmp.fa")
fn <- "tmp.fa"
pred <- getShape(fn)
}
\author{
Federico Comoglio
}
\keyword{core}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getShape.R
\name{getShape}
\alias{getShape}
\title{Predict DNA shape from a FASTA file}
\usage{
getShape(filename, shapeType = 'Default', parse = TRUE,
methylate = FALSE, methylatedPosFile = NULL)
}
\arguments{
\item{filename}{The name of input fasta format file, including
full path to file if it is located outside the current working directory.}

\item{shapeType}{A character indicating the shape parameters which can be
"MGW", "ProT", "Roll", "HelT" or "All" (meaning all four shapes)}

\item{parse}{A logical value indicating whether parse the prediction
result}

\item{methylate}{A logical value indicating wheter consider methlatation}

\item{methylatedPosFile}{The name of input postion file indicating
methlated position}
}
\value{
shapeList A List containing shapre prediction result
}
\description{
The DNA prediction uses a sliding pentamer window where structural features
unique to each of the 512 distinct pentamers define a vector of minor
groove width (MGW), Roll, propeller twist (ProT), and helix twist (HelT) at
each nucleotide position (Zhou, et al., 2013). MGW and ProT define
base-pair parameter whereas Roll and HelT represent base pair-step
parameters. The values for each DNA shape feature as function of its
pentamer sequence were derived from all-atom Monte Carlo simulations where
DNA structure is sampled in collective and internal degrees of freedom in
combination with explicit counter ions (Zhang, et al., 2014). The Monte
Carlo simulations were analyzed with a modified Curves approach
(Zhou, et al., 2013). Through data mining, average values for each shape
feature were calculated for the on average 44 occurrences of each pentamer
in an ensemble of Monte Carlo trajectories for 2,121 DNA fragments of 12-27
base pairs in length. DNAshapeR predicts four DNA shape features, which were
observed in various co-crystal structures playing an important role in
specific protein-DNA binding. The core prediction algorithm enables
ultra-fast, high-throughput predictions of shape features for thousands of
genomic sequences and is implemented in C++. Since it is likely that
features describing additional structural properties or equivalent features
derived from different experimental or computational sources will become
available, the package has a flexible modular design that easily allows
future expansions.
In the latest version, we further added additional 9 DNA shape features
beyond our previous set of 4 features, and expanded our available repertoire
to a total of 13 features, including 6 inter-base pair or base pair-step
parameters (HelT, Rise, Roll, Shift, Slide, and Tilt), 6 intra-base pair or
base pair-step parameters (Buckle, Opening, ProT, Shear, Stagger,
and Stretch), and MGW.
}
\details{
Predict biophysical feature

Our previous work explained protein-DNA binding specificity based on
correlations between MGW and electrostatic potential (EP) observed in
experimentally available structures (Joshi, et al., 2007). However, A/T
and C/G base pairs carry different partial charge distributions in the
minor groove (due primarily to the guanine amino group), which will affect
minor-groove EP. We developed a high-throughput method to predict
minor-groove EP based on data mining of results from solving the nonlinear
Poisson-Boltzmann calculations (Honig & Nicholls, 1995) on 2,297 DNA
structures derived from Monte Carlo simulations. DNAshapeR includes EP
as an additional feature.
}
\examples{
fn <- system.file("extdata", "CGRsample.fa", package = "DNAshapeR")
pred <- getShape(fn)
}
\author{
Federico Comoglio & Tsu-Pei Chiu
}
\keyword{core}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getShape.R
\name{convertMethFile}
\alias{convertMethFile}
\title{Convert fasta file to methylated file format}
\usage{
convertMethFile(fastaFileName, methPositionFileName)
}
\arguments{
\item{fastaFileName}{The name of the input fasta format file, including
full path to file if it is located outside the current working directory.}

\item{methPositionFileName}{The name of the input position file
indicating the methlation position}
}
\value{
methFileName fasta file containing methylated Cytosine
}
\description{
Convert fasta file to methylated file format
}
\author{
Satyanarayan Rao & Tsu-Pei Chiu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/encodeSeqShape.R
\name{normalize}
\alias{normalize}
\title{Min-Max normalization}
\usage{
normalize(x, max, min)
}
\arguments{
\item{x}{A matrix containing encoded features}

\item{max}{A number maximum number for Min-Max Normalization}

\item{min}{A number minimum number for Min-Max Normalization}
}
\value{
featureVector A matrix containing encoded features. shape feature is
represented as continuous numbers
}
\description{
Min-Max normalization
}
\author{
Tsu-Pei Chiu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/encodeSeqShape.R
\name{encodeNstOrderShape}
\alias{encodeNstOrderShape}
\title{Encode n-st order shape features
DNAshapeR can be used to generate feature vectors for a user-defined model.
The model can be a shape model. There are four structural parameters
including MGW, Roll, ProT and HelT. The second order shape features are
product terms of values for the same category of shape features at adjacent
positions.}
\usage{
encodeNstOrderShape(n, shapeMatrix, shapeType)
}
\arguments{
\item{n}{A number indicating n-st order shape encoding}

\item{shapeMatrix}{A matrix containing DNAshape prediction result}

\item{shapeType}{A character name of shape (MGW, Roll, ProT, HelT) features}
}
\value{
featureVector A matrix containing encoded features. shape feature is
represented as continuous numbers
}
\description{
Encode n-st order shape features
DNAshapeR can be used to generate feature vectors for a user-defined model.
The model can be a shape model. There are four structural parameters
including MGW, Roll, ProT and HelT. The second order shape features are
product terms of values for the same category of shape features at adjacent
positions.
}
\author{
Tsu-Pei Chiu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotShape.R
\name{heatShape}
\alias{heatShape}
\title{Plot heatmap of DNA shape features}
\usage{
heatShape(shapeMatrix, nBins, ordRow = NULL, useRaster = TRUE, ... )
}
\arguments{
\item{shapeMatrix}{A matrix containing DNAshape prediction results.}

\item{nBins}{An integer specifying the number of equally-sized bins in which shape predictions should be aggregated. Summarized predictions can be visualized by setting nBins=1.}

\item{ordRow}{A numeric vector (of the same length as the number of rows of shapeMatrix) defining the permutation of the rows of shapeMatrix to be used for plotting. Default to NULL, i.e. rows are ordered by coefficients of variation.}

\item{useRaster}{Logical, if TRUE a bitmap raster is used to plot the image instead of polygons (see ?graphics::image for details).}

\item{...}{Additional parameters to be passed to the image.plot function (see ?fields::image.plot for details).}
}
\value{
Called for its effects
}
\description{
Plot heatmap of DNA shape features
}
\examples{
fn <- system.file("extdata", "CGRsample.fa", package = "DNAshapeR")
pred <- getShape(fn)
library(fields)
heatShape(pred$MGW, 20)
}
\author{
Federico Comoglio
}
\keyword{core}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getShape.R
\name{readNonStandardFastaFile}
\alias{readNonStandardFastaFile}
\title{Read the position fasta file}
\usage{
readNonStandardFastaFile(filename)
}
\arguments{
\item{filename}{The name of the input position file
indicating the methlation position}
}
\value{
df dataframe
}
\description{
Read the position fasta file
}
\author{
Satyanarayan Rao & Tsu-Pei Chiu
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotShape.R
\name{plotShape}
\alias{plotShape}
\title{Plot metaprofiles of DNA shape features}
\usage{
plotShape(shapeMatrix, background = NULL,
colDots = rgb( 0, 0, 1, 0.1),
colDotsBg = rgb( 0, 0, 0, 0.1),
colLine = 'steelblue', colLineBg = 'gray50', cex = 0.5, lwd = 2, ylim, ...)
}
\arguments{
\item{shapeMatrix}{A matrix containing DNAshape prediction results}

\item{background}{A matrix containing DNAshape prediction results for a set of background regions. Default to NULL, i.e. background not provided.}

\item{colDots}{A character vector specifying the color of the points representing the column mean of shapeMatrix. Default to rgb( 0, 0, 1, 0.1).}

\item{colDotsBg}{A character vector specifying the color of the points representing the column mean of background. Default to rgb( 0, 0, 0, 0.1).}

\item{colLine}{A character string giving the color name of line representing the column mean of shapeMatrix. Default to 'steelblue'.}

\item{colLineBg}{A character string giving the color name of line representing the column mean of background. Default to 'gray50'.}

\item{cex}{A numerical value giving the amount by which plotting text and symbols should be magnified relative to the default. Default to 0.5.}

\item{lwd}{A numerical value specifying the line width. Default to 2.}

\item{ylim}{A numerical vector of size 2 specifying the y-axis plot range.}

\item{...}{Additional parameters to be passed to the R plot function.}
}
\value{
Called for its effects
}
\description{
DNA shape features can be visualized as
aggregated line plots (also known as metaprofiles, see Comoglio et al., 2015), heat maps (Yang et al., 2014) and
genome browser tracks (Chiu et al., 2014).
}
\examples{
fn <- system.file("extdata", "CGRsample.fa", package = "DNAshapeR")
pred <- getShape(fn)
plotShape(pred$MGW)
plotShape(pred$ProT)
plotShape(pred$Roll)
plotShape(pred$HelT)
}
\author{
Federico Comoglio
}
\keyword{core}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotShape.R
\name{trackShape}
\alias{trackShape}
\title{Plot track view of DNA shape features}
\usage{
trackShape( filename, shapeList )
}
\arguments{
\item{filename}{The name of the input fasta format file, including
full path to file if it is located outside the current working directory}

\item{shapeList}{A list containing four DNAshape prediction results}
}
\value{
Called for its effects
}
\description{
Plot track view of DNA shape features
}
\note{
None.
}
\examples{
fn2 <- system.file("extdata", "SingleSeqsample.fa", package = "DNAshapeR")
pred2 <- getShape(fn2)
trackShape(fn2, pred2) # Only for single sequence file
}
\author{
Tsu-Pei Chiu
}
\keyword{core}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getShape.R
\name{readShape}
\alias{readShape}
\title{Read (parse) DNA shape predictions}
\usage{
readShape(filename)
}
\arguments{
\item{filename}{character name of the file containing shape predictions, including
full path to file if it is located outside the current working directory.}
}
\value{
shapeMatrix matrix containing the shape prediction result
}
\description{
Read DNA shape predictions
}
\examples{
fn <- system.file("extdata", "CGRsample.fa", package = "DNAshapeR")
pred <- readShape(fn)
}
\author{
Federico Comoglio & Tsu-Pei Chiu
}
\keyword{core}
